' Sinusoszillator
'
' Sprache: Bascom-AVR-Basic Version 2.0.7.9 Demo/Freeware

' Erzeugung per PWM- Oszillator; sein Puls-Pausenverhaeltnis wird mit den 128 Werten einer approximierten
' Sinusfunktion geladen, so dass sich eine Treppe in Sinusform ergibt.

' Da die PWM- Taktfrequenz ueber 18 kHz betragen muss, damit man sie nicht hoert, muss der AtTiny 13 mit 9,6 MHz
' Takt betrieben werden.

' Der Attiny 13 kann jedoch auf 1,2 MHz geflasht bleiben (Auslieferungszustand), die Taktumschaltung auf 9,6
' MHz erfolgt ueber das CLKPR- Register zur Laufzeit des Programms

' Der Sinus liegt an OC0A, also PB0

' Der Generator wird mit einem Low an PB3 aktiviert (es wird also gegen Masse getastet)
'
' zur Unterdrueckung des Einschaltknacksens schwillt die Huellkurve des Sinus nach einer Quadrat-Funktion
' (Befehl "Shift M, Right Z") an bzw. ab. Nebenbei sorgt sie auch fuer das Enprellen des Tasteingangs (PB3)
'
' Die NF-Ausgangsfrequenz kann ueber ein Maeuseklavier an PB1 und PB2 in groben Stufen veraendert werden.
' PB1 und PB2 werden nur in den Tastpausen abgefragt, um die Tonerzeugung nicht "auszubremsen"
'
' geaendert: etwas haertere Hllkurve; statt 7 Shifts nur 3 (A=2), Hllkurve symmetriert (Aufruf Sinus verschoben)
'
' geaendert: Version _bat fuer Batteriebetrieb. Generator geht nach Ende Ton in Powerdown; Aufwecken per Pin Change Interrupt.
'
' geaendert: Pin Change Interrupt geht manchmal in die Gruetze. Fuehrt zu gelegentlichen Tonstoerungen.
' Haesslicher Workaround: Zaehlschleife sorgt dafuer, dass der Controller erst nach etwa 10 Sekunden in Powerdown geht.
' So wird er seltener aufgeweckt

' diese Version passt zu "Uli-Generator-Platine" mit Jumper an PB3/4



$regfile = "attiny13.dat"
$crystal = 9600000
$hwstack = 32
$swstack = 8
$framesize = 8

Dim A As Byte                                             ' fuer Verzoegerungsschleife, die zu niedrigerer Frequenz fuehrt
Dim B As Byte                                              ' speichert letzten Wert von A
Dim N As Byte                                              ' fuer die Zaehlschleife
Dim M As Byte                                              ' fuer PWM-Wert
Dim Z As Byte                                              ' fuer Huellkurve
Dim Offtimer As Word

Clkpr = 128                                                ' Aenderung Taktrate vorbereiten
Clkpr = 0                                                  ' 9,6 MHz
Clkpr = 0                                                  ' Wert bestaetigen

Acsr.7 = 1                                                 ' Analog-Komparator ausschalten, spart etwas Strom
Gimsk.5 = 1                                                 ' Pin Change Interrupt Enable
Pcmsk.1 = 1                                                 ' Pin Change Interrupt PB1 aktivieren
Sreg.7 = 1                                                  ' Interrupts global freigeben


Ddrb = &B0000_0001                                         ' alle Pins bis auf PB0 Eingang
Portb = &B0001_1110                                        ' alle Pins bis auf PB0 Pullup-Widerstand
Didr0 = &B0000_0101

Config Timer0 = Pwm , Prescale = 1 , Compare A Pwm = Clear Up
Start Timer0
Tccr0a.0 = 1                                               ' wgm00
Tccr0a.1 = 1                                               ' wgm01
Tccr0b.3 = 0                                               ' wgm02


Do

    If PinB.1 = 0 Then                                     ' Abfrage, ob Signal an PB3 low ist
        Offtimer = 0
        Gosub Sinus
        If Z > 0 Then
            Decr Z                                         ' Z startet mit 2 und wird nach jeder Sinsuwelle bis 0 vermindert
        End If                                             ' bewirkt Anschwellen der Sinuskurve
    Else
        If Z < 2 Then
            Incr Z                                         ' Z startet mit 0 und wird nach jeder Sinsuwelle bis 2 erhoeht
            Gosub Sinus                                    ' bewirkt Abschwellen der Sinuskurve
        Else
            A = PinB And &B00011000                            ' Schalter liegt an PB3 und PB4; mgliche Werte: 0, 8, 16, 24
            Shift A, Right, 3                              ' mgliche Werte fr A: 0,1,2,3
            A = 4 - A                                          ' nun Werte 5, 4, 3, 2
            Incr Offtimer
            Waitus 100
        End If
    End If

    If Offtimer > 64000 Then                              ' 640000: etwa 10-fache Laenge Word -> 10 sec. Wartezeit vor Powerdown
        Offtimer = 0
        Powerdown
    End If

Loop


Sinus:
    Restore Sinusdata                                      ' Zeiger auf Anfang der Data-Tabelle setzen
    For N = 1 To 128
        Read M
        Shift M , Right , Z                                ' bewirkt Ansteigen der Hllkurve
        Pwm0a = M                                          ' der Wert in PWM0A bestimmt das Puls- / Pausenverhltnis
        B = 0
        Do                                                 ' Verzgerungsschleife, dauert um so lnger, je groesser A ist
            Incr B
        Loop Until B = A
    Next N
Return




Sinusdata:

Data 0 , 0 , 0 , 1 , 2 , 3 , 5 , 7 , 9 , 12 , 14 , 17 , 21 , 24 , 28 , 32,
Data 36 , 41 , 46 , 51 , 56 , 61 , 66 , 72 , 78 , 83 , 89 , 95 , 101 , 107 , 114 , 120,
Data 127 , 133 , 139 , 145 , 151 , 157 , 163 , 169 , 175 , 181 , 186 , 192 , 197 , 202 , 207 , 212,
Data 216 , 221 , 225 , 228 , 232 , 235 , 238 , 241 , 244 , 246 , 248 , 250 , 251 , 252 , 253 , 253,
Data 253 , 253 , 253 , 252 , 251 , 250 , 248 , 246 , 244 , 241 , 239 , 236 , 232 , 229 , 225 , 221,
Data 216 , 212 , 207 , 202 , 197 , 192 , 187 , 181 , 175 , 169 , 164 , 158 , 151 , 145 , 139 , 133,
Data 127 , 120 , 114 , 108 , 102 , 96 , 90 , 84 , 78 , 72 , 67 , 61 , 56 , 51 , 46 , 41,
Data 37 , 33 , 28 , 25 , 21 , 18 , 15 , 12 , 9 , 7 , 5 , 3 , 2 , 1 , 0 , 0,
